package com.tzld.piaoquan.ad.engine.commons.score.model;

import com.tzld.piaoquan.ad.engine.commons.score.rankitem.VideoAdThompsonRankItem;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdActionFeature;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdRankItem;
import org.apache.commons.math3.distribution.BetaDistribution;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

public class VideoAdThompsonModel extends Model{
    private static final Logger LOGGER = LoggerFactory.getLogger(VideoAdThompsonModel.class);

    private Map<String, AdActionFeature> featureCache = new HashMap<>();
    private Map<Long, AdActionFeature> adActionFeatureMap = new HashMap<>();

    @Override
    public int getModelSize() {
        return featureCache.size();
    }

    @Override
    public boolean loadFromStream(InputStreamReader in) throws Exception {
        Map<String, AdActionFeature> initModel = new HashMap<>();
        Map<Long, AdActionFeature> tempAdActionFeatureMap =new HashMap<>();
        try (BufferedReader reader = new BufferedReader(in)) {
            String line = null;
            int size = 0;
            while ((line = reader.readLine()) != null) {
                String[] items = line.split("\t");
                if (items.length < 5) {
                    continue;
                }
//                headvideoid   STRING
//    ,advertiserid STRING
//    ,adid         STRING
//    ,creativeid   STRING
//    ,conversion   STRING
//    ,view         STRING
//    ,click        STRING
//    ,cpa          STRING
                double click=new BigDecimal(items[7].trim()).doubleValue();
                double conversion=new BigDecimal(items[7].trim()).doubleValue();
                double view= new BigDecimal(items[6].trim()).doubleValue();
                Long creativeId=new BigDecimal(items[4].trim()).longValue();
                AdActionFeature adActionFeature=tempAdActionFeatureMap.getOrDefault(creativeId,new AdActionFeature());
                adActionFeature.setAdClick(adActionFeature.getAdClick()+click);
                adActionFeature.setAdConversion(adActionFeature.getAdConversion()+conversion);
                adActionFeature.setAdView(adActionFeature.getAdView()+view);
                tempAdActionFeatureMap.put(creativeId,adActionFeature);
                this.putFeature(
                        initModel,
                        new BigDecimal(items[0].trim()).longValue(),
                        creativeId,
                        view,
                        click,
                        conversion
                );
                size++;
            }
            this.featureCache = initModel;
            this.adActionFeatureMap=tempAdActionFeatureMap;
            LOGGER.info("[UNION THOMPSON SAMPLING MODEL LOAD] mode load over, size {}", size);
        } catch (
                Exception e) {
            LOGGER.info("[UNION THOMPSON SAMPLING MODEL LOAD] mode load error ", e);
        } finally {
            in.close();
        }

        return true;
    }

    private void putFeature(Map<String, AdActionFeature> modelCache, Long videoId, Long creativeId, Double view, Double click, Double conversion) {
        AdActionFeature adActionFeature =  new AdActionFeature();
        adActionFeature.setAdView(view);
        adActionFeature.setAdClick(click);
        adActionFeature.setAdConversion(conversion);
        modelCache.put(videoId+"_"+creativeId, adActionFeature);

    }

    public double ctrScore(VideoAdThompsonRankItem item) {
        AdActionFeature adActionFeature = this.featureCache.getOrDefault(item.getVideoId()+"_"+item.getAdId(),  new AdActionFeature());
        if(adActionFeature.getAdView()<1000){
            adActionFeature=this.adActionFeatureMap.getOrDefault(item.getAdId(), new AdActionFeature());
        }
        int alpha_cvr = (int) adActionFeature.getAdClick() + 10;
        int beta_cvr = 100 + (int) adActionFeature.getAdView() - (int) adActionFeature.getAdClick();
        return this.betaSampler(alpha_cvr, beta_cvr);
    }

    public double cvrScore(VideoAdThompsonRankItem item) {
        AdActionFeature adActionFeature = this.featureCache.getOrDefault(item.getVideoId()+"_"+item.getAdId(),  new AdActionFeature());
        if(adActionFeature.getAdView()<1000){
            adActionFeature=this.adActionFeatureMap.getOrDefault(item.getAdId(), new AdActionFeature());
        }
        int alpha_cvr = (int) adActionFeature.getAdConversion() + 10;
        int beta_cvr = 100 + (int) adActionFeature.getAdClick() - (int) adActionFeature.getAdConversion();
        return this.betaSampler(alpha_cvr, beta_cvr);
    }

    public double betaSampler(double alpha, double beta) {
        BetaDistribution betaSample = new BetaDistribution(alpha, beta);
        return betaSample.sample();

    }
}
